function symbfct(P)
% SYMBFCT     - Symbolic factorization for sparse Cholesky factorization.
%            symbfct(P) does a minimum degree ordering and a symbolic
%            factorization for a symmetric square sparse matrix P, and 
%            passes the results as global variables.
%            It calls two Fortran MEX files.

% Yin Zhang, January, 1995
% Department of Mathematics and Statistics
% University of Maryland  Baltimore County

% PATCH by Jos F. Sturm, May 1998.
% 1. Use global variable "OUTFID" for sending messages. (NOT USED)
% 2. Don't call symfct.mex* if P is diagonal, since that would
%    cause a segmentation violation.

global XLNZ NNZL PERM INVP CACHE_SIZE
global XSUPER XLINDX LINDX SNODE SPLIT TMPSIZ

% ----- check input matrix P -----
m = size(P,1);
if size(P,2) ~= m | ~issparse(P) | isempty(P)
   error('Matrix must be square, sparse and nonempty.');
end;
Pdiag = diag(diag(P)); P = P - Pdiag;

% ----- min-degree ordering -----
t = cputime;
fprintf('  min-degree ordering ... ');
[PERM, INVP] = ordmmd(P);
fprintf('Done. CPU seconds: %g\n',cputime-t);

% ----- symbolic factorization -----
if nnz(P) > 0
  t = cputime;
  fprintf('  calling symfct.mex* ... ');
  [XLNZ,NNZL,XSUPER,XLINDX,LINDX,SNODE,SPLIT,TMPSIZ,PERM,INVP] = ...
                         symfct(P,PERM,INVP,CACHE_SIZE);
  fprintf('Done. CPU seconds: %g\n',cputime-t);
end
